Class: Solana::Ruby::Kit::RpcTypes::ClusterUrl

Inherits:
T::Struct
  • Object
show all
Extended by:
T::Sig
Defined in:
lib/solana/ruby/kit/rpc_types/cluster_url.rb

Overview

A cluster-tagged URL string. Mirrors TypeScript’s branded string types:

MainnetUrl, DevnetUrl, TestnetUrl, ClusterUrl

In Ruby we carry the cluster tag as a symbol on a wrapper struct, since Ruby cannot brand primitive String values.

Instance Method Summary collapse

Instance Method Details

#to_sObject



24
# File 'lib/solana/ruby/kit/rpc_types/cluster_url.rb', line 24

def to_s = @url