Class: Solana::Ruby::Kit::TransactionIntrospection::LoadedAddresses

Inherits:
T::Struct
  • Object
show all
Defined in:
lib/solana/ruby/kit/transaction_introspection/loaded_addresses.rb

Overview

Loaded ALT addresses as returned by getTransaction's meta.loadedAddresses. The two arrays are kept in the same order the runtime uses to resolve instruction account indices. Mirrors LoadedAddresses from @solana/transaction-introspection.