Module: Solana::Ruby::Kit::Transactions

Extended by:
T::Sig
Defined in:
lib/solana/ruby/kit/transactions/compiler.rb,
lib/solana/ruby/kit/transactions/transaction.rb

Defined Under Namespace

Classes: FullySignedTransaction, Transaction

Constant Summary collapse

TRANSACTION_SIZE_LIMIT =

Maximum wire-encoded transaction size in bytes (Solana protocol limit).

T.let(1232, Integer)
TransactionMessageBytes =

The wire-encoded bytes of a compiled transaction message. Mirrors TypeScript’s ‘TransactionMessageBytes` (branded Uint8Array).

T.type_alias { String }
SignaturesMap =

An ordered map of signer addresses to their Ed25519 signatures (or nil when the address has been reserved for signing but not yet signed). Mirrors TypeScript’s ‘SignaturesMap = OrderedMap<Address, SignatureBytes | null>`.

T.type_alias { T::Hash[String, T.nilable(String)] }

Class Method Summary collapse

Class Method Details

.assert_fully_signed_transaction!(transaction) ⇒ Object



60
61
62
63
64
65
66
67
68
# File 'lib/solana/ruby/kit/transactions/transaction.rb', line 60

def assert_fully_signed_transaction!(transaction)
  missing = transaction.signatures.filter_map { |addr, sig| addr if sig.nil? }
  return if missing.empty?

  Kernel.raise SolanaError.new(
    :SOLANA_ERROR__TRANSACTION__SIGNATURES_MISSING,
    addresses: missing
  )
end

.assert_sendable_transaction!(transaction) ⇒ Object



101
102
103
104
# File 'lib/solana/ruby/kit/transactions/transaction.rb', line 101

def assert_sendable_transaction!(transaction)
  assert_fully_signed_transaction!(transaction)
  assert_within_size_limit!(transaction)
end

.assert_within_size_limit!(transaction) ⇒ Object



80
81
82
83
84
85
86
87
88
# File 'lib/solana/ruby/kit/transactions/transaction.rb', line 80

def assert_within_size_limit!(transaction)
  actual = wire_encode_transaction(transaction).bytesize
  return if actual <= TRANSACTION_SIZE_LIMIT

  Kernel.raise SolanaError.new(
    SolanaError::TRANSACTIONS__EXCEEDS_SIZE_LIMIT,
    { actual_size: actual, limit: TRANSACTION_SIZE_LIMIT }
  )
end

.compile_transaction_message(message) ⇒ Object



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
# File 'lib/solana/ruby/kit/transactions/compiler.rb', line 50

def compile_transaction_message(message)
  Kernel.raise SolanaError.new(:SOLANA_ERROR__TRANSACTION__FEE_PAYER_MISSING) if message.fee_payer.nil?
  fee_payer = T.must(message.fee_payer)

  # ── 1. Collect accounts and merge roles ────────────────────────────────
  # Insertion-ordered hash: address_str → merged AccountRole integer.
   = T.let({}, T::Hash[String, Integer])

  # Fee payer is always the first writable signer.
  [fee_payer.value] = Instructions::AccountRole::WRITABLE_SIGNER

  message.instructions.each do |ix|
    # The instruction's program address is a readonly non-signer participant.
    prog = ix.program_address.value
    [prog] ||= Instructions::AccountRole::READONLY

    (ix.accounts || []).each do |meta|
      addr     = meta.address.value
      existing = [addr] || Instructions::AccountRole::READONLY
      [addr] = Instructions::AccountRole.merge(existing, meta.role)
    end
  end

  # ── 2. Partition into the four groups and sort within each ─────────────
  fp = fee_payer.value

  writable_signers    = T.let([], T::Array[String])
  readonly_signers    = T.let([], T::Array[String])
  writable_non_signers = T.let([], T::Array[String])
  readonly_non_signers = T.let([], T::Array[String])

  .each do |addr, role|
    next if addr == fp  # fee payer is handled separately

    is_signer   = Instructions::AccountRole.signer_role?(role)
    is_writable = Instructions::AccountRole.writable_role?(role)

    if is_signer && is_writable
      writable_signers << addr
    elsif is_signer
      readonly_signers << addr
    elsif is_writable
      writable_non_signers << addr
    else
      readonly_non_signers << addr
    end
  end

  writable_signers.sort!
  readonly_signers.sort!
  writable_non_signers.sort!
  readonly_non_signers.sort!

  # Fee payer first, then writable signers, readonly signers, non-signers.
  ordered = [fp] + writable_signers + readonly_signers +
            writable_non_signers + readonly_non_signers

  # ── 3. Build index lookup ──────────────────────────────────────────────
   = T.let({}, T::Hash[String, Integer])
  ordered.each_with_index { |addr, i| [addr] = i }

  # ── 4. Message header (3 bytes) ────────────────────────────────────────
  num_required_sigs    = 1 + writable_signers.size + readonly_signers.size
  num_readonly_signed  = readonly_signers.size
  num_readonly_unsigned = readonly_non_signers.size

  header = [num_required_sigs, num_readonly_signed, num_readonly_unsigned].pack('CCC').b

  # ── 5. Account addresses section ───────────────────────────────────────
  accounts_section = encode_compact_u16(ordered.size)
  ordered.each do |addr_str|
    accounts_section = accounts_section + Addresses.decode_address(Addresses::Address.new(addr_str))
  end

  # ── 6. Recent blockhash (32 bytes) ─────────────────────────────────────
  # Mirrors upstream compile-transaction.ts #581:
  #   - BlockhashLifetime → use the blockhash bytes
  #   - DurableNonceLifetime → use the nonce value bytes (stored in the same field)
  #   - No lifetime → 32 zero bytes (placeholder; must be replaced before signing)
  constraint = message.lifetime_constraint
  blockhash_bytes =
    if constraint.is_a?(TransactionMessages::BlockhashLifetimeConstraint)
      Addresses.decode_address(Addresses::Address.new(constraint.blockhash))
    elsif constraint.is_a?(TransactionMessages::DurableNonceLifetimeConstraint)
      Addresses.decode_address(Addresses::Address.new(constraint.nonce))
    else
      ("\x00" * 32).b
    end

  # ── 7. Instructions section ────────────────────────────────────────────
  ixs_section = encode_compact_u16(message.instructions.size)

  message.instructions.each do |ix|
    prog_idx    = T.must([ix.program_address.value])
    ix_accounts = ix.accounts || []
    ix_indices  = ix_accounts.map { |m| T.must([m.address.value]) }
    data_bytes  = (ix.data || '').b

    ixs_section = ixs_section +
                  [prog_idx].pack('C').b +
                  encode_compact_u16(ix_indices.size) +
                  ix_indices.pack('C*').b +
                  encode_compact_u16(data_bytes.bytesize) +
                  data_bytes
  end

  message_bytes = (header + accounts_section + blockhash_bytes + ixs_section).b

  # ── 8. Signatures map (one nil slot per required signer) ───────────────
  signer_addresses = [fp] + writable_signers + readonly_signers
  signatures = T.let({}, T::Hash[String, T.nilable(String)])
  signer_addresses.each { |addr| signatures[addr] = nil }

  Transaction.new(message_bytes: message_bytes, signatures: signatures)
end

.fully_signed_transaction?(transaction) ⇒ Boolean

Returns:

  • (Boolean)


53
54
55
# File 'lib/solana/ruby/kit/transactions/transaction.rb', line 53

def fully_signed_transaction?(transaction)
  transaction.signatures.values.all? { |sig| !sig.nil? }
end

.get_signature_from_transaction(transaction) ⇒ Object



43
44
45
46
47
48
# File 'lib/solana/ruby/kit/transactions/transaction.rb', line 43

def get_signature_from_transaction(transaction)
  sig_bytes = transaction.signatures.values.first
  Kernel.raise SolanaError.new(:SOLANA_ERROR__TRANSACTION__FEE_PAYER_SIGNATURE_MISSING) unless sig_bytes

  Keys.encode_signature(Keys::SignatureBytes.new(sig_bytes))
end

.get_transaction_message_size(message) ⇒ Object



21
22
23
# File 'lib/solana/ruby/kit/transactions/compiler.rb', line 21

def get_transaction_message_size(message)
  compile_transaction_message(message).message_bytes.bytesize
end

.partially_sign_transaction(signing_keys, transaction) ⇒ Object



118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
# File 'lib/solana/ruby/kit/transactions/transaction.rb', line 118

def partially_sign_transaction(signing_keys, transaction)
  new_signatures  = transaction.signatures.dup
  unexpected      = T.let([], T::Array[String])

  signing_keys.each do |signing_key|
    verify_key = signing_key.verify_key
    addr_str   = Addresses.encode_address(verify_key.to_bytes)

    unless new_signatures.key?(addr_str)
      unexpected << addr_str
      next
    end

    sig_bytes = Keys.sign_bytes(signing_key, transaction.message_bytes)
    existing  = new_signatures[addr_str]

    next if existing && existing == sig_bytes.value

    new_signatures[addr_str] = sig_bytes.value
  end

  if unexpected.any?
    Kernel.raise SolanaError.new(
      :SOLANA_ERROR__TRANSACTION__ADDRESSES_CANNOT_SIGN_TRANSACTION,
      expected_addresses:    transaction.signatures.keys,
      unexpected_addresses:  unexpected
    )
  end

  Transaction.new(
    message_bytes: transaction.message_bytes,
    signatures:    new_signatures
  )
end

.sendable_transaction?(transaction) ⇒ Boolean

Returns:

  • (Boolean)


94
95
96
# File 'lib/solana/ruby/kit/transactions/transaction.rb', line 94

def sendable_transaction?(transaction)
  fully_signed_transaction?(transaction) && within_size_limit?(transaction)
end

.sign_transaction(signing_keys, transaction) ⇒ Object



161
162
163
164
165
166
167
168
169
# File 'lib/solana/ruby/kit/transactions/transaction.rb', line 161

def sign_transaction(signing_keys, transaction)
  signed = partially_sign_transaction(signing_keys, transaction)
  assert_fully_signed_transaction!(signed)

  FullySignedTransaction.new(
    message_bytes: signed.message_bytes,
    signatures:    signed.signatures
  )
end

.wire_encode_transaction(transaction) ⇒ Object



178
179
180
181
182
183
184
# File 'lib/solana/ruby/kit/transactions/compiler.rb', line 178

def wire_encode_transaction(transaction)
  sigs    = transaction.signatures
  header  = encode_compact_u16(sigs.size)
  sig_bytes = sigs.values.map { |s| (s || ("\x00" * 64)).b }.join

  (header + sig_bytes + transaction.message_bytes).b
end

.within_size_limit?(transaction) ⇒ Boolean

Returns:

  • (Boolean)


73
74
75
# File 'lib/solana/ruby/kit/transactions/transaction.rb', line 73

def within_size_limit?(transaction)
  wire_encode_transaction(transaction).bytesize <= TRANSACTION_SIZE_LIMIT
end