Class: Solana::Ruby::Kit::Instructions::AccountMeta
- Inherits:
-
T::Struct
- Object
- T::Struct
- Solana::Ruby::Kit::Instructions::AccountMeta
- Defined in:
- lib/solana/ruby/kit/instructions/accounts.rb
Overview
Metadata for a single account referenced by an instruction. Mirrors TypeScript’s ‘AccountMeta<TAddress>`.
Combines an address with an AccountRole that declares whether the account is a signer, writable, or both.