Module: Solana::Ruby::Kit::Rpc::Api
- Defined in:
- lib/solana/ruby/kit/rpc/api/get_slot.rb,
lib/solana/ruby/kit/rpc/api/get_balance.rb,
lib/solana/ruby/kit/rpc/api/get_block_time.rb,
lib/solana/ruby/kit/rpc/api/get_epoch_info.rb,
lib/solana/ruby/kit/rpc/api/get_transaction.rb,
lib/solana/ruby/kit/rpc/api/request_airdrop.rb,
lib/solana/ruby/kit/rpc/api/get_account_info.rb,
lib/solana/ruby/kit/rpc/api/get_block_height.rb,
lib/solana/ruby/kit/rpc/api/send_transaction.rb,
lib/solana/ruby/kit/rpc/api/get_vote_accounts.rb,
lib/solana/ruby/kit/rpc/api/get_epoch_schedule.rb,
lib/solana/ruby/kit/rpc/api/is_blockhash_valid.rb,
lib/solana/ruby/kit/rpc/api/get_inflation_reward.rb,
lib/solana/ruby/kit/rpc/api/get_latest_blockhash.rb,
lib/solana/ruby/kit/rpc/api/get_program_accounts.rb,
lib/solana/ruby/kit/rpc/api/simulate_transaction.rb,
lib/solana/ruby/kit/rpc/api/get_multiple_accounts.rb,
lib/solana/ruby/kit/rpc/api/get_signature_statuses.rb,
lib/solana/ruby/kit/rpc/api/get_token_account_balance.rb,
lib/solana/ruby/kit/rpc/api/get_signatures_for_address.rb,
lib/solana/ruby/kit/rpc/api/get_token_accounts_by_owner.rb,
lib/solana/ruby/kit/rpc/api/get_minimum_balance_for_rent_exemption.rb
Defined Under Namespace
Modules: GetAccountInfo, GetBalance, GetBlockHeight, GetBlockTime, GetEpochInfo, GetEpochSchedule, GetInflationReward, GetLatestBlockhash, GetMinimumBalanceForRentExemption, GetMultipleAccounts, GetProgramAccounts, GetSignatureStatuses, GetSignaturesForAddress, GetSlot, GetTokenAccountBalance, GetTokenAccountsByOwner, GetTransaction, GetVoteAccounts, IsBlockhashValid, RequestAirdrop, SendTransaction, SimulateTransaction Classes: LatestBlockhash, SignatureStatus
Constant Summary collapse
- EpochInfo =
Struct returned by get_epoch_info.
T.let( Struct.new( :absolute_slot, # Integer — current slot :block_height, # Integer :epoch, # Integer — current epoch :slot_index, # Integer — slot within current epoch :slots_in_epoch, # Integer — total slots in epoch :transaction_count, # Integer | nil keyword_init: true ), T.untyped )
- VoteAccountInfo =
Vote account info struct.
T.let( Struct.new( :vote_pubkey, :node_pubkey, :activated_stake, :epoch_vote_account, :commission, :last_vote, :epoch_credits, :root_slot, keyword_init: true ), T.untyped )
- EpochSchedule =
Struct returned by get_epoch_schedule.
T.let( Struct.new( :first_normal_epoch, # Integer — first full-length epoch after warmup :first_normal_slot, # Integer — first slot after warmup period :leader_schedule_slot_offset, # Integer — slots before epoch to compute leader schedule :slots_per_epoch, # Integer — max slots per epoch :warmup, # Boolean — whether epochs start short and grow keyword_init: true ), T.untyped )
- InflationReward =
Struct for a single entry in the getInflationReward response. nil entries indicate the address was not found or had no reward.
T.let( Struct.new( :amount, # Integer (Lamports) — reward credited :commission, # Integer — vote account commission at reward time :effective_slot, # Integer — slot in which rewards were delivered :epoch, # Integer — epoch for which reward occurred :post_balance, # Integer (Lamports) — post-reward account balance keyword_init: true ), T.untyped )
- SignatureInfo =
Struct representing one transaction signature entry returned by getSignaturesForAddress. Mirrors TypeScript’s GetSignaturesForAddressTransaction.
T.let( Struct.new( :block_time, # Integer | nil — Unix timestamp of block production, nil if unavailable :confirmation_status, # Symbol | nil — :processed, :confirmed, or :finalized :err, # Hash | nil — transaction error, nil if succeeded :memo, # String | nil — memo associated with the transaction :signature, # String — base-58 encoded transaction signature :slot, # Integer — slot containing the transaction :transaction_index, # Integer | nil — 0-based index within block (Agave 4.0+) keyword_init: true ), T.untyped )